-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLSkills.com
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- Make sure you set the directory appropriately!

CREATE DATABASE [TestFGDB]
	ON PRIMARY 
	(NAME = N'TestFGDBData', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql.1\mssql\data\TestFGDB2Data.MDF' , 
	SIZE = 10,
	FILEGROWTH = 10%,
	MAXSIZE = 15)
LOG ON 
	(NAME = N'TestFGDBLog', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql.1\mssql\data\TestFGDBLog.LDF' , 
	SIZE = 10, FILEGROWTH = 10%)
go

ALTER DATABASE [TestFGDB] ADD FILEGROUP [fg] 
go

-- Adding a file requires the same settings as the file's settings used in create database.
ALTER DATABASE [TestFGDB] 
	ADD FILE
		(NAME = N'TestFGDBFile1', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql.1\mssql\data\TestFGDBFile1.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fg]
go

ALTER DATABASE [TestFGDB] 
	ADD FILE
		(NAME = N'TestFGDBFile2', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql.1\mssql\data\TestFGDBFile2.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fg]
go

ALTER DATABASE [TestFGDB] 
	ADD FILE
		(NAME = N'TestFGDBFile3', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql.1\mssql\data\TestFGDBFile3.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fg]
go

-- Change over the to newly created database and verify the files:
USE TestFGDB
go

sp_helpfile
go

SET NOCOUNT ON
go

DROP TABLE Test_NewID
go

DROP TABLE Test_Ident
go

CREATE TABLE Test_NewID
(
	col1	uniqueidentifier	CONSTRAINT Test_NewID_Guid
						DEFAULT newid()
					CONSTRAINT Test_NewID_PK
						PRIMARY KEY CLUSTERED,
	col2	int	identity,
	col3	datetime default getdate(),
	col4	nvarchar(80) default host_name() + '\' + suser_name() + '\' + user_name(),
	col5	char(80) default 'This is a test. This is a test. This is a test. This is a test. This is a test.'
) ON FG
go

DECLARE @RowCounter	int,
	@StartTime		datetime
SELECT @StartTime = getdate(), @RowCounter = 1
WHILE @RowCounter <= 100000
BEGIN
	INSERT Test_NewID DEFAULT VALUES
	SELECT @RowCounter = @RowCounter + 1
END
SELECT DATEDIFF(ms, @StartTime, getdate()) AS [NewID TotalTime (ms)]
go

CREATE TABLE Test_NewSeqID
(
	col1	uniqueidentifier	CONSTRAINT Test_NewSeqID_Guid
						DEFAULT newsequentialid()
					CONSTRAINT Test_NewSeqID_PK
						PRIMARY KEY CLUSTERED,
	col2	int	identity,
	col3	datetime default getdate(),
	col4	nvarchar(80) default host_name() + '\' + suser_name() + '\' + user_name(),
	col5	char(80) default 'This is a test. This is a test. This is a test. This is a test. This is a test.'
) ON FG
go

DECLARE @RowCounter	int,
	@StartTime		datetime
SELECT @StartTime = getdate(), @RowCounter = 1
WHILE @RowCounter <= 100000
BEGIN
	INSERT Test_NewSeqID DEFAULT VALUES
	SELECT @RowCounter = @RowCounter + 1
END
SELECT DATEDIFF(ms, @StartTime, getdate()) AS [NewID TotalTime (ms)]
go

CREATE TABLE Test_Ident
(
	col1	uniqueidentifier	CONSTRAINT Test_Ident_Guid
						DEFAULT newid(),
	col2	int	identity	CONSTRAINT Test_Ident_PK
						PRIMARY KEY CLUSTERED,
	col3	datetime default getdate(),
	col4	nvarchar(80) default host_name() + '\' + suser_name() + '\' + user_name(),
	col5	char(80) default 'This is a test. This is a test. This is a test. This is a test. This is a test.'
) ON FG
go
DECLARE @RowCounter	int,
	@StartTime		datetime
SELECT @StartTime = getdate(), @RowCounter = 1
WHILE @RowCounter <= 100000
BEGIN
	INSERT Test_Ident DEFAULT VALUES
	SELECT @RowCounter = @RowCounter + 1
END
SELECT DATEDIFF(ms, @StartTime, getdate()) AS [Ident TotalTime (ms)]
go

-- SQL 2000
-- NewID Time: 52486
-- Ident Time: 47560
-- Percent of time: 90.61% of the time
select convert(decimal, 47560)/52486
-- Percent better: 10.3%
select convert(decimal, 52486)/47560

-- SQL 2005
-- NewID Time: 82436
-- NewSeqID Time: 48500
-- Ident Time: 51402
-- Percent of time: 62.35% of the time
select convert(decimal, 51402)/82436
-- Percent better: 60.32%
select convert(decimal, 82436)/51402

go
EXEC sp_spaceused Test_NewID, TRUE
EXEC sp_spaceused Test_Ident, TRUE
go

DBCC SHOWCONTIG(Test_NewID)
go
-- DBCC SHOWCONTIG scanning 'Test_NewID' table...
-- Table: 'Test_NewID' (453576654); index ID: 1, database ID: 12
-- TABLE level scan performed.
-- - Pages Scanned................................: 2079
-- - Extents Scanned..............................: 262
-- - Extent Switches..............................: 2075
-- - Avg. Pages per Extent........................: 7.9
-- - Scan Density [Best Count:Actual Count].......: 12.52% [260:2076]
-- - Logical Scan Fragmentation ..................: 49.78%
-- - Extent Scan Fragmentation ...................: 0.76%
-- - Avg. Bytes Free per Page.....................: 2468.3
-- - Avg. Page Density (full).....................: 69.50%
-- DBCC execution completed. If DBCC printed error messages, contact your system administrator.

DBCC SHOWCONTIG(Test_NewSeqID)
go
--DBCC SHOWCONTIG scanning 'Test_NewSeqID' table...
--Table: 'Test_NewSeqID' (117575457); index ID: 1, database ID: 11
--TABLE level scan performed.
--- Pages Scanned................................: 2326
--- Extents Scanned..............................: 294
--- Extent Switches..............................: 293
--- Avg. Pages per Extent........................: 7.9
--- Scan Density [Best Count:Actual Count].......: 98.98% [291:294]
--- Logical Scan Fragmentation ..................: 0.69%
--- Extent Scan Fragmentation ...................: 0.68%
--- Avg. Bytes Free per Page.....................: 56.4
--- Avg. Page Density (full).....................: 99.30%
--DBCC execution completed. If DBCC printed error messages, contact your system administrator.

DBCC SHOWCONTIG(Test_Ident)
go
-- DBCC SHOWCONTIG scanning 'Test_Ident' table...
-- Table: 'Test_Ident' (597577167); index ID: 1, database ID: 12
-- TABLE level scan performed.
-- - Pages Scanned................................: 1450
-- - Extents Scanned..............................: 183
-- - Extent Switches..............................: 182
-- - Avg. Pages per Extent........................: 7.9
-- - Scan Density [Best Count:Actual Count].......: 99.45% [182:183]
-- - Logical Scan Fragmentation ..................: 0.00%
-- - Extent Scan Fragmentation ...................: 0.55%
-- - Avg. Bytes Free per Page.....................: 27.0
-- - Avg. Page Density (full).....................: 99.67%
-- DBCC execution completed. If DBCC printed error messages, contact your system administrator.
go
